"""
config_reader.py
"""
import configparser
from dataclasses import dataclass
from typing import Optional, List, Tuple
from pathlib import Path

def read_config(file_path: Path) -> configparser.ConfigParser:
    """
    读取并解析指定路径的 INI 配置文件。

    参数:
        file_path (Path): 配置文件的路径。

    返回:
        configparser.ConfigParser: 解析后的配置对象。
    """
    config = configparser.ConfigParser()
    config.read(file_path, encoding='utf-8')
    return config

def config_get(config: configparser.ConfigParser, section: str, key: str,
               default: str = None) -> str:
    """
    从配置文件中获取指定键的值，支持默认值。

    如果在指定的 section 中找不到键，则尝试从默认 section 中获取。

    如果最终值为None则会抛出异常

    参数:
        config (configparser.ConfigParser): 配置对象。
        section (str): 要查询的 section 名称。
        key (str): 要查询的键名称。
        default (str): 最终配置文件没有时硬编码的默认值。

    返回:
        str: 查询到的值或默认值。
    """
    res = config.get(section, key, fallback=
                      config.get(config.default_section, key, fallback=default))
    # if is_raise and res is None:
    #     raise ValueError(f"Key '{key}' not found in section '{section}' or "\
    #                      "default section or default value.")
    return res

@dataclass
class NginxConfig:
    """
    表示一个 Nginx 配置项的数据类。

    属性:
        name (str): 配置项名称，默认为 "unknown"。
        host (str): 主机地址，默认为 "localhost"。
        port (int): 端口号，默认为 80。
        output_path (Optional[str]): 输出路径，默认为 None。
        tcp_nopush (str): TCP 推送选项，默认为 "off"。
        sendfile (str): Sendfile 选项，默认为 "off"。
        client_max_body_size (str): 客户端最大请求体大小，默认为 "100m"。
        proxy_connect_timeout (str): 代理连接超时时间，默认为 "60s"。
        proxy_send_timeout (str): 代理发送超时时间，默认为 "60s"。
        proxy_read_timeout (str): 代理读取超时时间，默认为 "60s"。
        web_socket_proxy (bool): 是否启用 WebSocket 代理，默认为 False。
    """
    name: str = "unknown"
    host: str = "localhost"
    port: int = 80
    output_path: Optional[str] = None
    tcp_nopush: str = "off"
    sendfile: str = "off"
    client_max_body_size: str = "100m"
    proxy_connect_timeout: str = "60s"
    proxy_send_timeout: str = "60s"
    proxy_read_timeout: str = "60s"
    web_socket_proxy: bool = False

def nginx_config(file_path: Path) -> Tuple[List[NginxConfig], dict]:
    """
    读取 INI 配置文件并解析为 NginxConfig 对象列表。

    参数:
        file_path (Path): 配置文件的路径。

    返回:
        Tuple[List[NginxConfig], dict]: 包含 NginxConfig 对象的列表和其他相关信息的字典。
    """
    config = read_config(file_path)
    configs = []

    g_section = config.default_section
    def _config_get(name, default = None):
        return config_get(config, g_section, name, default)

    for section in config.sections():
        g_section = section
        configs.append(NginxConfig(
            name=section,
            host=_config_get('host'),
            port=int(_config_get('port')),  # 将端口转换为整数类型
            web_socket_proxy=_config_get('web_socket_proxy') == 'True',
            client_max_body_size=_config_get('client_max_body_size',
                                             NginxConfig.client_max_body_size)
        ))
    return (configs, {})

@dataclass
class DockerComposeConfig:
    """
    表示一个 Docker Compose 配置项的数据类。

    属性:
        name (str): 服务名称。
        container_name (str): 容器名称。
        image (str): 使用的镜像。
        ports (Optional[list[str]]): 映射的端口列表，默认为 None。
        volumes (Optional[list[str]]): 挂载的卷列表，默认为 None。
        environment (Optional[list[str]]): 环境变量列表，默认为 None。
        extra_hosts (Optional[list[str]]): 额外的主机映射列表，默认为 None。
        export (Optional[str]): 导出的端口，默认为 None。
        network (Optional[str]): 所属网络，默认为 None。
    """
    name: str
    container_name: str
    image: str
    ports: Optional[List[str]] = None
    volumes: Optional[List[str]] = None
    environment: Optional[List[str]] = None
    extra_hosts: Optional[List[str]] = None
    export: Optional[str] = None
    network: Optional[str] = None

def docker_config(file_path: Path) -> Tuple[List[DockerComposeConfig], dict]:
    """
    读取 INI 配置文件并解析为 DockerComposeConfig 对象列表。

    参数:
        file_path (Path): 配置文件的路径。

    返回:
        Tuple[List[DockerComposeConfig], dict]: 包含 DockerComposeConfig 对象的列表和其他相关信息的字典。
    """
    config = read_config(file_path)
    configs = []

    g_section = ''
    def _config_get(name, default = None):
        return config_get(config, g_section, name, default)

    def _get_item(name) -> List[str]:
        """将逗号分隔的字符串转换为列表，并去除空格和空字符串。"""
        return [i.strip() for i in _config_get(name, '').split(',') if i.strip()]

    network = set()
    for section in config.sections():
        g_section = section
        configs.append(DockerComposeConfig(
            name=section,
            container_name=_config_get('container_name'),
            image=_config_get('image'),
            volumes=_get_item('volumes'),
            ports=_get_item('ports'),
            environment=_get_item('environment'),
            extra_hosts=_get_item('extra_hosts'),
            export=_config_get('export'),
            network=_config_get('network'),
        ))
        network.add(_config_get('network'))
    return (configs, {'networks': list(network)})

def docker2nginx(file_path: Path) -> Tuple[List[NginxConfig], dict]:
    """
    将 Docker Compose 服务配置转换为 Nginx 代理配置。

    参数:
        file_path (Path): 配置文件的路径。

    返回:
        Tuple[List[NginxConfig], dict]: 包含 NginxConfig 对象的列表和其他相关信息的字典。
    """
    config = read_config(file_path)
    configs = []

    g_section = ''
    def _config_get(name):
        return config_get(config, g_section, name)

    def _get_item(name) -> List[str]:
        """将逗号分隔的字符串转换为列表，并去除空格和空字符串。"""
        return [i.strip() for i in _config_get(name).split(',') if i.strip()]

    for section in config.sections():
        if section == 'nginx':
            continue
        g_section = section
        configs.append(NginxConfig(
            name=config.get(section, 'nginx_name', fallback=section),
            host=_config_get('container_name'),
            port=_config_get('export'),
        ))
    return (configs, {})
